#!/usr/bin/env python
# coding: utf-8

# In[1]:


get_ipython().run_line_magic('load_ext', 'nb_black')
import os

import seaborn as sns
import matplotlib
import matplotlib.pyplot as plt
import pandas as pd
import scipy.stats as stats
import numpy as np
import statsmodels.api as sm
import statsmodels.formula.api as smf
from sklearn import preprocessing

# -*- coding: utf-8 -*-


# In[2]:


# PM2.5 components to include in analysis

component_list = [
    "Ammonium Ion PM2.5 LC",
    "Antimony PM2.5 LC",
    "Aluminum PM2.5 LC",
    "Arsenic PM2.5 LC",
    "Barium PM2.5 LC",
    "Bromine PM2.5 LC",
    "Calcium PM2.5 LC",
    "Cadmium PM2.5 LC",
    "Cesium PM2.5 LC",
    "Chromium PM2.5 LC",
    "Cobalt PM2.5 LC",
    "Copper PM2.5 LC",
    "Chloride PM2.5 LC",
    "Chlorine PM2.5 LC",
    "Indium PM2.5 LC",
    "Iron PM2.5 LC",
    "Lead PM2.5 LC",
    "Magnesium PM2.5 LC",
    "Manganese PM2.5 LC",
    "Mercury PM2.5 LC",
    "Molybdenum PM2.5 LC",
    "Nickel PM2.5 LC",
    "Phosphorus PM2.5 LC",
    "Potassium PM2.5 LC",
    "Rubidium PM2.5 LC",
    "Selenium PM2.5 LC",
    "Silicon PM2.5 LC",
    "Silver PM2.5 LC",
    "Sodium PM2.5 LC",
    "Strontium PM2.5 LC",
    "Sulfate PM2.5 LC",
    "Sulfur PM2.5 LC",
    "Tin PM2.5 LC",
    "Titanium PM2.5 LC",
    "Total Nitrate PM2.5 LC",
    "Vanadium PM2.5 LC",
    "Yttrium PM2.5 LC",
    "Zinc PM2.5 LC",
    "Zirconium PM2.5 LC",
]


# In[7]:


# Load smoke and fire distance analysis dataset

# df = pd.read_csv(
#    "C:\\Users\\RRice\\OneDrive - Environmental Protection Agency (EPA)\\naaqs-fires data\\smoke plume and fire distance analysis 2006-2018 California PM Spec BR\\2006-2018 HMS smoke and fire analysis PM2.5 SPEC CA 5-18-2021.csv"
# )

df = pd.read_csv("2006-2018 HMS smoke and fire analysis PM2.5 SPEC CA 11-3-2021.csv")
df["Date Local"] = pd.to_datetime(df["Date Local"])

# subset to parameters of interest

df = df.loc[df["Parameter Name"].isin(component_list)]

# subset to April-December

df = df.loc[df["Date Local"].dt.month.isin([4, 5, 6, 7, 8, 9, 10, 11, 12])]

# convert to km

df["closest fire km"] = df["closest fire"] / 1000

# create year column

df["Year"] = df["Date Local"].dt.year

# set negative measurements to 0

df.loc[df["Arithmetic Mean"] < 0, "Arithmetic Mean"] = 0

# make string values for the in smoke plume field

df["in_smoke?"] = df["in_smoke?"].map({0: "Not in smoke", 1: "In smoke"})

# use sklearn robust scalar to center and scale the data at each site for each year

df["robustscalar"] = df.groupby(
    ["State Code", "County Code", "Site Number", "POC", "Parameter Name", "Year"]
)["Arithmetic Mean"].transform(
    lambda x: preprocessing.robust_scale(
        x,
        with_centering=True,
        with_scaling=True,
        quantile_range=(25.0, 75.0),
        copy=True,
        unit_variance=False,
    )
)

# subset to measurements in a smoke plume with a closest fire within 500 km
df = df.loc[(df["in_smoke?"] == "In smoke") & (df["closest fire km"] <= 500)]

# rename columns to add underscores to closest_fire_km for quantreg function...

df.columns = [
    "State Code",
    "County Code",
    "Site Number",
    "POC",
    "Method Code",
    "Method Name",
    "Local Site Name",
    "Date Local",
    "Sample Duration",
    "Parameter Name",
    "Arithmetic Mean",
    "closest fire",
    "in_smoke?",
    "closest_fire_km",
    "Year",
    "robustscalar",
]


# In[8]:


# fit quantile regression model for each parameter

all_models = []
all_res = []

for param in component_list:

    data = df[df["Parameter Name"] == param].dropna(
        subset=["closest_fire_km", "robustscalar"], how="any"
    )

    mod = smf.quantreg("robustscalar ~ closest_fire_km", data)
    res = mod.fit(q=0.5)
    print(res.summary())
    all_res.append(res)

    quantiles = np.arange(0.02, 0.99, 0.01)

    def fit_model(q):
        res = mod.fit(q=q)
        return [
            q,
            res.params["Intercept"],
            res.params["closest_fire_km"],
        ] + res.conf_int().loc["closest_fire_km"].tolist()

    models = [fit_model(x) for x in quantiles]
    models = pd.DataFrame(models, columns=["q", "a", "b", "lb", "ub"])

    x = np.arange(data.closest_fire_km.min(), data.closest_fire_km.max(), 50)
    get_y = lambda a, b: a + b * x

    models["param"] = param
    all_models.append(models)


# In[14]:


# export model output to csv

pd.concat(all_models).to_csv("fire_distance_quantreg_summary/all_models.csv")

# print out model summaries

# for i, x in enumerate(all_res):
#    print(x.summary(title=component_list[i]))
#    print("\n")


# In[12]:


# Create plot displaying slope of regression across quantiles for each parameter


def plot_threshold(q):
    modeldf = pd.concat(all_models)

    modeldf["param"] = modeldf["param"].str.rstrip(" PM2.5 LC")

    # subset to qth percentile

    sortm = modeldf.loc[modeldf["q"].round(2) == q].sort_values("b")

    # filter to components with 95th percentile 95% CI above or below 0 (95th percentile slope sig. different from 0)

    sortm["sig"] = (sortm["ub"] < 0) | (sortm["lb"] > 0)

    sortm = sortm.loc[sortm["sig"] == True]

    # create 2 groups based on 95th percentile slope

    sortm["Response group"] = pd.qcut(sortm["b"], 2, labels=["high", "low"])

    # join sort and subset fields to model dataframe

    modeldf = (
        modeldf.set_index("param")
        .join(sortm.set_index("param")[["Response group", "sig"]])
        .reset_index()
    )

    palette = sns.cubehelix_palette(n_colors=17, start=2.8, rot=0.1)

    # fig, ax = plt.subplots(figsize=(10, 6))

    # 2x2 facet
    g = sns.FacetGrid(
        modeldf.loc[(modeldf["sig"] == True)].sort_values(
            ["Response group", "param"]
        ),  # subset to above quantile 0.5 and variables with group assigned
        col="Response group",
        height=6,
        col_wrap=2,
        sharex=True,
    )

    fig = g.map_dataframe(
        sns.lineplot,
        x="q",
        y="b",
        hue="param",
        # palette="Paired_r",
    )

    for ax in g.axes.ravel():
        ax.legend(fontsize=12)

    g.fig.subplots_adjust(top=0.9)  # adjust the Figure in rp

    g.set_axis_labels(
        "Quantile of PM₂.₅ Component\nChemical Concentration",
        r"Regression Slope",
        fontsize=16,
    )
    g.tight_layout()

    axes = g.axes.flatten()
    axes[0].set_title("High Response", fontsize=16)
    axes[1].set_title("Low Response", fontsize=16)

    fig.savefig(
        f"quantreg slope 11-3-2021 {str(q)}.png",
        facecolor="w",
        transparent=False,
        pad_inches=0.2,
        dpi=300,
    )
    return modeldf


# In[13]:


# plot significant components slope, using 80th percentile as 95% confidence significance threshold

plot_threshold(0.8)


# In[ ]:




