# 1-27-2021 BR geospatial analysis code for naaqs-fires project
# python version 3.6.12

import pandas as pd
import geopandas
import os
import shapely.speedups
import numpy as np

shapely.speedups.enable()


def preprocess_aqs_data(aqs_csv_path):

    # aqs_df input is standard csv from AQS read into pandas dataframe

    aqs_df = pd.read_csv(aqs_csv_path)

    # get aqs data into geopandas
    nad83 = aqs_df[aqs_df.Datum == "NAD83"]
    wgs84 = aqs_df[aqs_df.Datum == "WGS84"]

    # make geopandas dataframes with appropriate crs
    nad83_gdf = geopandas.GeoDataFrame(
        nad83,
        geometry=geopandas.points_from_xy(nad83.Longitude, nad83.Latitude),
        crs="EPSG:4269",
    )
    wgs84_gdf = geopandas.GeoDataFrame(
        wgs84,
        geometry=geopandas.points_from_xy(wgs84.Longitude, wgs84.Latitude),
        crs="EPSG:4326",
    )

    # reproject nad83 to wgs84
    nad83_gdf.to_crs("EPSG:4326", inplace=True)

    # recombine datasets
    aqs_gdf = pd.concat([nad83_gdf, wgs84_gdf], ignore_index=True).drop(columns="Datum")

    # datetime formatting

    aqs_gdf["Date Local"] = pd.to_datetime(aqs_gdf["Date Local"])

    return aqs_gdf


def preprocess_fire_points(hms_fire_folder_path_list):

    # load fire points and put them in one dataframe
    dflist = []

    for i in hms_fire_folder_path_list:
        try:
            df = geopandas.read_file(i)
            df["filename"] = os.path.basename(i)
            dflist.append(df)
        except Exception as e:
            print(e)

    fires_df = pd.concat(dflist)

    # datetime formatting
    fires_df["YearDay"] = pd.to_datetime(
        fires_df.YearDay, format="%Y%j", errors="coerce"
    )

    # datetime formatting
    fires_df["date_from_filename"] = pd.to_datetime(
        fires_df.filename.str.extract("(\d+)", expand=False).astype(int),
        format="%Y%m%d",
    )

    # use date from filename for analysis

    fires_df["Date"] = fires_df["date_from_filename"]

    fires_df.crs = "EPSG:4326"  # set WGS84 as datum

    return fires_df


def preprocess_smoke_polygons(hms_smoke_folder_path_list):

    # load smoke polygons and put them in one dataframe
    dflist = []

    for i in hms_smoke_folder_path_list:

        try:
            df = geopandas.read_file(i)
            df["filename"] = os.path.basename(i)
            dflist.append(df)

        except Exception as e:
            print(e)

    smoke_df = pd.concat(dflist)

    # datetime formatting
    smoke_df["date"] = pd.to_datetime(
        smoke_df.filename.str.extract("(\d+)", expand=False).astype(int),
        format="%Y%m%d",
    )
    smoke_df.crs = "EPSG:4326"

    return smoke_df


def smoke_plume_analysis(aqs_gdf, hms_smoke_gdf):

    # get list of unique dates included in aqs data
    dates = (
        pd.to_datetime(aqs_gdf["Date Local"]).dt.strftime("%Y%m%d").unique().tolist()
    )

    exportlist = []

    for date in dates:

        try:
            # subset hms smoke dataframe to day

            smokedf = hms_smoke_gdf[hms_smoke_gdf.date == date]

            # dissolve smoke into one polygon
            smokedf["dissolve_column"] = date
            smokedf = smokedf.dissolve(by="dissolve_column").reset_index()

            # filter aqs df to date of interest

            aqs_gdf_date = aqs_gdf.loc[aqs_gdf["Date Local"] == date, :]

            # check if aqs points are within dissolved daily smoke polygon
            aqs_gdf_date["in_smoke?"] = aqs_gdf_date.within(smokedf.loc[0, "geometry"])

            # export

            exportlist.append(aqs_gdf_date)

        except Exception as e:

            # Mark missing smoke days with NA values

            aqs_gdf_date = aqs_gdf.loc[aqs_gdf["Date Local"] == date, :]

            aqs_gdf_date["in_smoke?"] = np.nan

            # export

            exportlist.append(aqs_gdf_date)

    return pd.concat(exportlist)


def distance_to_nearest_fire(aqs_gdf, fire_points_gdf):

    # aqs_gdf input is geopandas dataframe of data with crs assigned to "EPSG:4326"

    # fire_points_gdf is HMS fires shapefiles concatenated into one geopanda dataframe with crs set to "EPSG:4326" (WGS84)

    # reproject each dataset to NAD83/California Albers (units in meters)

    aqs_gdf = aqs_gdf.to_crs("EPSG:3310")
    fire_points_gdf = fire_points_gdf.to_crs("EPSG:3310")

    # calculate distance to closest fire

    def distance_to_closest_point(df):

        # closest fire to Pb measurement on each day in year

        return (
            fire_points_gdf[fire_points_gdf.Date == df["Date Local"]]
            .geometry.distance(df.geometry)
            .min()
        )

    aqs_gdf["closest fire"] = aqs_gdf.apply(distance_to_closest_point, axis=1)

    return aqs_gdf
