#> Import data from permutation indices
setwd("C:/Users/KBoaggio/OneDrive - Environmental Protection Agency (EPA)/Wildfire data") 
library(permute)
library(plyr)
set.seed(123454321)

#> Read in data with aluminum data for all years and all sites.
aq.dat.og <- read.csv("2006-2018_pm_speciation_labeled_new_11_3_21.csv",header=T)

#> Make Type a factor
aq.dat.og$Type <- factor(aq.dat.og$Type,levels=c("No Fire", "Fire"))

#> Create a unique site id (SS - CCC - NNNN - PPP)
aq.dat.og$site.id <- paste0(sprintf("%02d",aq.dat.og$State.Code),sprintf("%03d",aq.dat.og$County.Code),
                            sprintf("%04d",aq.dat.og$Site.Number),sprintf("%03d",aq.dat.og$POC))

setwd("C:/Users/KBoaggio/OneDrive - Environmental Protection Agency (EPA)/Wildfire data/RData files - all fires") #rerun_large_file") 
load(file=paste0("Potassium PM2.5 LC_updated_Data_2006-2018_Permute_Data_11_15.Rdata"))
setwd("C:/Users/KBoaggio/OneDrive - Environmental Protection Agency (EPA)/Wildfire data") 
aq.dat.in <- aq.dat.og[grepl('Potassium PM2.5 LC', aq.dat.og$Parameter.Name, ignore.case=TRUE),]
fire.dat <- aq.dat.in[grepl('^Fire$', aq.dat.in$Type),]
n.samples <- 10000

#> For each year, calculate the metric of interest.  This is the average across sites of the
#> difference in the average concentration on Fire days minus the average concentration on Non Fire days.  
#> Then use the permuted indices to shuffle the data at each site 
#> (we are not shuffling data across sites), calculating a new metric for each permuted sample. 
#> Finally calculate the p-value for the year by counting how many of the permuted samples had a 
#> difference value that was greater than or equal to the difference value in the original dataset. 
#> Save these p-values from each year in the vector called collect.pvalues.

#> Time the loop.
ptm0 <- proc.time()

collect.pvalues <- NULL
for(YYYY in 2006:2018){
  print(paste("Calculating p-value for year", YYYY))
  
  aq.dat.sub <- subset(aq.dat.in,Year==YYYY)
  
  #> Drop  sites that do not have any fire days. 
  table.fire.data <- as.data.frame(table(aq.dat.sub$site.id,aq.dat.sub$Type))
  drop.sites <- as.character(table.fire.data$Var1[table.fire.data$Freq==0])
  aq.dat <- subset(aq.dat.sub, !site.id%in%drop.sites)
  
  #> Sort the dataset by site.id
  aq.dat.sort <- aq.dat[order(aq.dat$site.id),]
  
  #> Calculate average concentration for Fire days and No Fire days using the tapply function.
  avg.conc.orig <- tapply(aq.dat.sort$Arithmetic.Mean,list(type=aq.dat.sort$Type,site=aq.dat.sort$site.id),mean,na.rm=T)
  #> Calculate the difference in concentration between Fire days and No Fire days using the original dataset. 
  avg.diff.orig <- mean(avg.conc.orig[1,]-avg.conc.orig[2,],na.rm=T) # 2. 1
  
  #> Retrieve the matrix of permuted indicies for this year that
  #> was created in the previous loop. 
  shuffle.days.sample <- get(paste0("shuffle.days.sample.",YYYY))
  
  avg.diff.permute <- NULL
  for(i in 1:n.samples){
    shuffle.days.i <- shuffle.days.sample[,i]
    avg.conc.i <- tapply(aq.dat.sort$Arithmetic.Mean[shuffle.days.i],list(type=aq.dat.sort$Type,site=aq.dat.sort$site.id),mean,na.rm=T)
    avg.diff.permute <- rbind(avg.diff.permute, mean(avg.conc.i[c("No Fire"),]-avg.conc.i[c("Fire"),],na.rm=T)) ###
  }
  
  p.val <- sum(avg.diff.permute>avg.diff.orig,na.rm=T)/sum(!is.na(avg.diff.permute))
  collect.pvalues <- rbind(collect.pvalues,data.frame(year=YYYY,pvalue=p.val))
  
  remove(aq.dat,aq.dat.sort,shuffle.days.sample,avg.conc.orig,avg.diff.orig,p.val)
}

print(((proc.time() - ptm0)[3])/60)
#2.7 min ~.45 min a year

species <- aq.dat.in$Parameter.Name[1]
units <- aq.dat.in$Units.of.Measure[1]

#> Boxplots of Al concentrations on Fire days and No Fire days, by year, annotated with p-values from 
#> permutation test.  
jpeg("Potassium_lower_boxplots_w_pvalues_12_15.jpeg",height=500,width=800)
aq.dat.fire <- subset(aq.dat.in,Type=="Fire")
aq.dat.nofire <- subset(aq.dat.in,Type=="No Fire")
boxplot(aq.dat.nofire$Arithmetic.Mean~aq.dat.nofire$Year,at=(1:13)-.2,xlim=c(0,13),boxwex=.4,pch=16,cex=.5,xaxt="n",xlab="",
        main=paste0("Distribution of ",species," by year"),	ylab=paste0(units))
boxplot(aq.dat.fire$Arithmetic.Mean~aq.dat.fire$Year,at=(1:13)+.2,boxwex=.4,col=2,add=2,pch=16,cex=.5,border="darkred",xaxt="n")
axis(side=1,at=1:13,labels=2006:2018)
text(1:13,rep(-0.5,13),labels=sprintf("%.3f",collect.pvalues$pvalue),font=2)
which.signif.1 <- collect.pvalues$pvalue < .1
which.signif.05 <- collect.pvalues$pvalue < .05
text(c(1:13)[which.signif.1],rep(-0.5,length(which.signif.1)),labels=sprintf("%.3f",collect.pvalues$pvalue)[which.signif.1],col=3,font=2)
text(c(1:13)[which.signif.05],rep(-0.5, length(which.signif.05)),labels=sprintf("%.3f",collect.pvalues$pvalue)[which.signif.05],col="purple",font=2)
text(0,-0.5,labels="p value =",font=2)
legend("topright",fill=c("grey",2),legend=c("No fire event","Fire event"))
dev.off()

#> Save p-values to csv file
